/*
 * REVA Puls v2  
 * You can use this freely as long as you act friendly to humans and animals!
 */

#include <PWM.h>  //Arduino PWM Frequency library - https://code.google.com/archive/p/arduino-pwm-frequency-library/

//OLED BEGIN
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>

#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 32 // OLED display height, in pixels

// Declaration for an SSD1306 display connected to I2C (SDA, SCL pins)
#define OLED_RESET    -1// 4 // Reset pin # (or -1 if sharing Arduino reset pin)
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#define DUTY_UP       A3  //SW7
#define DUTY_DOWN     A2  //SW8
#define FRQ_DOWN      A1  //SW6
#define FRQ_UP        A0  //SW5

#define PWM_MAX_DUTY  65535
#define PWM_INIT_DUTY 6000
#define PWM_MIN_DUTY  655 

#define PWM_MAX_FRQ   120000
#define PWM_INIT_FRQ  20400
#define PWM_MIN_FRQ   1

int32_t pwm_duty;
int32_t pwm_frequency;
int pwm_pin = 9;
bool initialiced = 0;
int freqstep = 10;
int dutystep = 100;

void setup() {

  // initialize the serial communication:
  Serial.begin(9600);

  //OLED BEGIN
  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { // Address 0x3C for 128x32
    Serial.println(F("SSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }
  // Show initial display buffer contents on the screen --
  // the library initializes this with an Adafruit splash screen.
  display.display();

  pinMode(DUTY_UP, INPUT_PULLUP);
  pinMode(DUTY_DOWN, INPUT_PULLUP);
  pinMode(FRQ_UP, INPUT_PULLUP);
  pinMode(FRQ_DOWN, INPUT_PULLUP);

  //initialize all timers except for 0, to save time keeping functions
  InitTimersSafe();

  pwm_duty = PWM_INIT_DUTY;
  pwm_frequency = PWM_INIT_FRQ;

  pwmWriteHR(pwm_pin, pwm_duty);
  //pwmWrite(pwm_pin, pwm_duty);
  SetPinFrequencySafe(pwm_pin,pwm_frequency); 

  Serial.println("REVA v1.24");
  Serial.println("PWM Frequency / PWM Duty Cycle");
  Serial.print(pwm_frequency);
  Serial.print("/");
  Serial.println(pwm_duty);

  OledShowVersion();
  OledRefresh();

  DDRB = 0x20; // will set digital pin 13 as outputpin
}

void loop(){  

if (initialiced == 0)

{
  SetPinFrequencySafe(pwm_pin,pwm_frequency);
  PORTB = 0x20; // will set digital pin 13 to HIG
  pwmWriteHR(pwm_pin, pwm_duty);
  initialiced = 1;            
}


while(1) {
    while(!(digitalRead(DUTY_UP)) && pwm_duty < PWM_MAX_DUTY){
      pwm_duty = (pwm_duty + dutystep);
      pwmWriteHR(pwm_pin, pwm_duty);
      PORTB = 0x20;
      OledRefresh();      
      SerialRefresh();
      delay(100);
    }
    while(!(digitalRead(DUTY_DOWN)) && pwm_duty > PWM_MIN_DUTY){
      pwm_duty = (pwm_duty - dutystep);   
      pwmWriteHR(pwm_pin, pwm_duty);
      PORTB = 0x20;
      OledRefresh();      
      SerialRefresh();
      delay(100);
    }
  
    while(!(digitalRead(FRQ_UP)) && pwm_frequency < PWM_MAX_FRQ) {
      pwm_frequency = (pwm_frequency + freqstep);
      SetPinFrequencySafe(pwm_pin,pwm_frequency);
      pwmWriteHR(pwm_pin, pwm_duty);
      PORTB = 0x20;
      OledRefresh();
      SerialRefresh();
      delay(100);
    }
    while(!(digitalRead(FRQ_DOWN)) && pwm_frequency > PWM_MIN_FRQ) {
      pwm_frequency = (pwm_frequency - freqstep);
      SetPinFrequencySafe(pwm_pin,pwm_frequency);
      pwmWriteHR(pwm_pin, pwm_duty);
      PORTB = 0x20;
      OledRefresh();
      SerialRefresh();      
      delay(100);
    }
  }
}

void SerialRefresh(void) {
  Serial.print(pwm_frequency);
  Serial.print("/");
  Serial.println(pwm_duty);
}

void OledShowVersion(void) {
  display.clearDisplay();
  display.setTextSize(2); // Draw 2X-scale text
  display.setTextColor(SSD1306_WHITE);
  display.setCursor(0, 0);
  display.print(F("REVA 1.24"));
  display.display();      // Show initial text
  delay(2000);
}
 
void OledRefresh(void) {
  display.clearDisplay();
  display.setTextSize(2);
  display.setCursor(0, 0);
  display.setTextColor(SSD1306_WHITE);
  display.println(pwm_frequency,DEC);
  display.print(pwm_duty,DEC);
  display.display();      // Show initial text
}
